/*
 * (c) Copyright 2002 by Rob Braun
 * All rights reserved.  The file named COPYRIGHT specifies the terms
 * and conditions for redistribution.
 */

#include "config.h"
#ifdef XINETD_DYNAMIC

#include <stdlib.h>
#include "xmodule.h"
#include "xdl.h"

#ifdef HAVE_DLFCN_H
#include <dlfcn.h>
#endif

#ifdef HAVE_MACHO_DYLD_H
#include <mach-o/dyld.h>
#endif

void *xdlopen(const char *mod_path)
{
#ifdef HAVE_DLFCN_H
	return dlopen(mod_path, RTLD_NOW);
#elif defined(HAVE_MACHO_DYLD_H)
	NSObjectFileImage ofile;
	void *ret = NULL;
	if( NSCreateObjectFileImageFromFile(mod_path, &ofile) != 
			NSObjectFileImageSuccess )
		return NULL;
	ret = NSLinkModule(ofile, mod_path, NSLINKMODULE_OPTION_PRIVATE | 
		NSLINKMODULE_OPTION_BINDNOW);
	NSDestroyObjectFileImage(ofile);
	return ret;
#endif
}

servicefn xdlsym(void *handle, const char *symbol)
{
#ifdef HAVE_DLFCN_H
	return (servicefn)dlsym(handle, symbol);
#elif defined(HAVE_MACHO_DYLD_H)
	NSSymbol nsSymbol;
	char *_symbol;
	
	if( symbol == NULL )
		return NULL;
	_symbol = malloc( strlen(symbol) + 2 );
	if( _symbol == NULL )
		return NULL;
	strcpy(_symbol, "_");
	strcat(_symbol, symbol);
	
	nsSymbol = NSLookupSymbolInModule(handle, _symbol);
	if( nsSymbol == NULL )
		return NULL;
	return (servicefn)NSAddressOfSymbol(nsSymbol);
#endif
}

void xdlclose(void *handle)
{
#ifdef HAVE_DLFCN_H
	dlclose(handle);
#elif defined(HAVE_MACHO_DYLD_H)
	NSUnLinkModule((NSModule)handle, NSUNLINKMODULE_OPTION_NONE);
#endif
	return;
}

#endif /* XINETD_DYNAMIC */
